#include "udpwhois.h" 

void main(int argc, char *argv[])
{
 struct sockaddr_in client,server;
 struct hostent *host_entry;


 if(argc != 3)
 {
  cout<<"Usage: ./program_name <server name> <port number>"<<endl;
  exit(0); 
 }
 char* server_name = argv[1];
 int port = atoi(argv[2]); 
 unsigned int server_size = sizeof(client);

 int sock = socket(AF_INET,SOCK_DGRAM,IPPROTO_UDP);
 verify(sock,sock,socket_flag);

 if((host_entry=gethostbyname(server_name))==NULL)
   perror("ERROR: gethostbyname has failed\n");
 
 server.sin_family = AF_INET;
 server.sin_port = htons((unsigned short)port);
 server.sin_addr.s_addr = *((unsigned long *)host_entry->h_addr);

 client.sin_family = AF_INET;
 client.sin_port = htons(0);
 client.sin_addr.s_addr=htonl(INADDR_ANY);

 MESSAGE msg;
 strcpy(msg.msg,"hello");

 verify(sock,
        bind(sock,(struct sockaddr *)&client,sizeof(client)),
        bind_flag); 

 verify(sock,
        sendto(sock, &msg, sizeof(msg),0,
              (struct sockaddr *)&server, sizeof(server)),
        send_flag);

 unsigned int none = 0;

 verify(sock,
        recvfrom(sock, &msg, sizeof(msg),0,
                (struct sockaddr *)&client, &none),
        receive_flag);
 cout<<"Client receives: "<<msg.msg<<endl;
 closeConnection(sock); 
}


/**
 * verify's method is to verify the success of the operation.
 * if the operation fails, it will terminate the whole program.
 * @param  int sockfd contains socket descriptor if available
 *         int byte contains the byte receives from the operation
 *         int flag contains the description of the flag
 * @return none
 */
void verify(int sockfd, int byte, int flag)
{
 switch(flag)
 {
  case 0:if(byte<0)
         {
           perror("ERROR: socket has failed\n");
           exit(0); 
         }
         break;
  case 1:if(byte<0) 
         {
           perror("ERROR: bind has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 2:if(byte<0)
         {
           perror("ERROR: listen has failed\n");
           closeConnection(sockfd);
           exit(0);
         } 
         break;
  case 3:if(byte<0)
         {
           perror("ERROR: accept has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 4:if(byte<0)
         {
           perror("ERROR: send has failed\n");
           closeConnection(sockfd);
           exit(0);
         } 
         break;
  case 5:if(byte<0)
         {
           perror("ERROR: receive has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 6:if(byte<0)
         {
           perror("ERROR: pipe has failed\n");
           exit(0);
         }
         break; 
  case 7:if(byte<0)
         {
           perror("ERROR: write has failed\n");
           exit(0);
         }
         break;
  case 8:if(byte<0)
         {
           perror("ERROR: read has failed\n");
           exit(0);
         }
         break;
  case 9:if(byte<0)
         {
           perror("ERROR: select has failed\n");
           exit(0);
         }
  default: break;
 }
}

/**
 * closeConnection's method is to close the socket
 * descriptor that contains the connection of 
 * the client/server's model
 * @param  int sockfd contains socket descriptor
 * @return none
 */
void closeConnection(int sockfd)
{
 close(sockfd);
}
