#include "proxy_server.h" 


void main()
{
  int masterfd[2];
  int slave[4][2];

  close(masterfd[WRITE]);

  int identification,pid;
  for(identification=0; identification<4; identification++)
  {
   if(pipe(slave[identification]) < 0)
   {
    cerr<<"error in pipe"<<endl;
    exit(0); 
   }
   pid = fork();
   if(pid < 0)
   {
    cerr<<"error in fork"<<endl;
    exit(0);
   }
   close(slave[identification][READ]);
   if(pid == 0)
     break;
  }
  if(identification == 4)
    cout<<"Parent finished"<<endl;

  if(pid > 0)
  {
   
   close(masterfd[WRITE]);
   MESSAGE pmsg;
  }
  else if(pid == 0)
  {
   cout<<"child: "<<identification<<endl;
   close(masterfd[READ]);
   close(slave[identification][READ]);

   for(int j=0; j<4; j++)
   {
    if(identification != j)
    {
     close(slave[j][READ]);
     close(slave[j][WRITE]);
    }
   }
   MESSAGE cmsg;
   cmsg.rec = 1;
   write(masterfd,&cmsg,sizeof(cmsg)); 
   close(slave[identification][READ]);
   close(slave[identification][WRITE]);
   exit(0);
  } 
  else
  {
   cout<<"You should not be here"<<endl;
   exit(0);
  } 
}

/**
 * verify's method is to verify the success of the operation.
 * if the operation fails, it will terminate the whole program.
 * @param  int sockfd contains socket descriptor if available
 *         int byte contains the byte receives from the operation
 *         int flag contains the description of the flag
 * @return none
 */
void verify(int sockfd, int byte, int flag)
{
 switch(flag)
 {
  case 0:if(byte<0)
         {
           perror("ERROR: socket has failed\n");
           exit(0); 
         }
         break;
  case 1:if(byte<0) 
         {
           perror("ERROR: bind has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 2:if(byte<0)
         {
           perror("ERROR: listen has failed\n");
           closeConnection(sockfd);
           exit(0);
         } 
         break;
  case 3:if(byte<0)
         {
           perror("ERROR: accept has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 4:if(byte<0)
         {
           perror("ERROR: send has failed\n");
           closeConnection(sockfd);
           exit(0);
         } 
         break;
  case 5:if(byte<0)
         {
           perror("ERROR: receive has failed\n");
           closeConnection(sockfd);
           exit(0);
         }
         break;
  case 6:if(byte<0)
         {
           perror("ERROR: pipe has failed\n");
           exit(0);
         }
         break; 
  case 7:if(byte<0)
         {
           perror("ERROR: write has failed\n");
           exit(0);
         }
         break;
  case 8:if(byte<0)
         {
           perror("ERROR: read has failed\n");
           exit(0);
         }
         break;
  case 9:if(byte<0)
         {
           perror("ERROR: select has failed\n");
           exit(0);
         }
  default: break;
 }
}

/**
 * closeConnection's method is to close the socket
 * descriptor that contains the connection of 
 * the client/server's model
 * @param  int sockfd contains socket descriptor
 * @return none
 */
void closeConnection(int sockfd)
{
 close(sockfd);
}

/**
 * sendStruct's method is to send MESSAGE struct to the client/
 * server that is address by the socket descriptor
 * @param  int sockfd contains socket descriptor
 *         MESSAGE msg contains the message struct
 * @return -1 if fail otherwiser return number of bytes send
 */
int sendStruct(int sockfd, MESSAGE msg)
{
 int numSend = 0;
 numSend = send(sockfd,(char *)&msg, sizeof(MESSAGE),0);
 return numSend;
}

/**
 * receivedStruct's method is to receive struct from
 * the given socket. It will return -1 if it fails
 * to receive otherwise it will return int bigger than
 * zero.
 * @param  int sockfd contains sock descriptor
 *         struct MSG contains the struct itself
 * @return int 
 */
int receivedStruct(int sockfd, MESSAGE& receivedMsg)
{
  int byteRecv = 0;
  byteRecv = recv(sockfd,(char *)&receivedMsg,sizeof(MESSAGE),0);
  return byteRecv;
}

/**
 * openConnection's method is to establish connection
 * between the client and host. 
 * @param  int PORT contains predefine port number
 * @return sockfd if flag == 1 else connfd is return
 */
int openConnection(int PORT)
{
 int servsize;
 int yes = 1;
 int sockfd,connfd,clientsize;
 struct sockaddr_in my_add;
 struct sockaddr_in other_add;

 sockfd = socket(AF_INET, SOCK_STREAM,IPPROTO_TCP);
 verify(0,sockfd,socket_flag);

 bzero(&(my_add.sin_zero),sizeof(my_add));
 my_add.sin_family = AF_INET;
 my_add.sin_port = htons(PORT);
 my_add.sin_addr.s_addr = INADDR_ANY;
 
 verify(sockfd,        
        bind(sockfd,(struct sockaddr *)&my_add, 
        sizeof(struct sockaddr)),
        bind_flag);

 verify(sockfd,listen(sockfd,BACKLOG),listen_flag);
 
 return sockfd;
}

